var StripChart = {
    amplitude: 0,   // half of peak to peak
    currentValue: 0, // slider position
    prevValue: 0,  // previous position
    
    xSlideBase: 0,
    ySlideBase: 0,
    
    draggable: false, // can slider be dragged?
    
    indicatorColor: "#000", // color for indicator and line
    
    labelAmplitude: false,    
    WIDTH: 180,    // dimensions of actual stripchart
    HEIGHT: 100,
    LEFT_MARGIN: 20,
    TOP_MARGIN: 20,
    
    canvas: null,
    ctx: null,
    
    running: null, // the timer for moving chart
    vLineCount: 0, // vertical line counter
    VLINE_SEP: 45, // # of pixels between vertical separators
    
    init: function(canvasId, settings) {
        this.canvas = document.getElementById(canvasId);
        this.ctx = this.canvas.getContext("2d");
        this.xSlideBase = this.LEFT_MARGIN + this.WIDTH - 35;
        this.ySlideBase = this.TOP_MARGIN + this.HEIGHT / 2;
        Object.assign(this, settings);
        
        if (this.draggable) {
            this.canvas.onmousedown = this.downFunction.bind(this);
            
            this.canvas.onmouseup = function(evt) {
                evt.target.onmousemove = null;
            }
        }
    },
    
    downFunction: function(evt) {
        evt.target.onmousemove = this.dragFunction.bind(this);
        this.dragFunction(evt);
    },
    
    dragFunction: function(evt) {
        var x = evt.offsetX;
        var y = evt.offsetY;
        if ((x >= this.LEFT_MARGIN + this.WIDTH - 10) && (x <= this.LEFT_MARGIN + this.WIDTH + 10)) {
            this.currentValue = Math.min(this.amplitude, Math.max(-this.amplitude, this.ySlideBase - y));
        }
    },

    
    chartY: function(yValue) {
        yValue = Math.min(this.amplitude, Math.max(-this.amplitude, yValue));
        return this.ySlideBase - (this.HEIGHT  / 2.0) * (yValue / this.amplitude);
    },
    
    drawChartInterval: function() {
        if (this.vLineCount % this.VLINE_SEP == 0) {
            drawLine(this.ctx, this.LEFT_MARGIN + this.WIDTH, this.TOP_MARGIN,
                     this.LEFT_MARGIN + this.WIDTH, this.TOP_MARGIN + this.HEIGHT);
        }
    },
    
    syncValues: function(value) {
        this.prevValue = value;
        this.currentValue = value;
    },

    draw: function() {
        this.ctx.lineWidth = 1;
        this.ctx.strokeStyle = "#666";
        this.ctx.fillStyle = "#fff";
        this.ctx.beginPath;
        this.ctx.rect(this.LEFT_MARGIN - 1, this.TOP_MARGIN - 1, this.WIDTH + 2, this.HEIGHT + 2);
        this.ctx.fill();
        this.ctx.stroke();
        
        // horizontal axis
        this.ctx.strokeStyle = "lightgray";
        drawLine(this.ctx, this.LEFT_MARGIN, this.ySlideBase, this.LEFT_MARGIN + this.WIDTH, this.ySlideBase);
        
        this.drawChartInterval();
        
        if (this.labelAmplitude) {
            this.ctx.fontStyle="12px sans-serif";
            this.ctx.fillStyle = "#fff"
            this.ctx.fillRect(this.LEFT_MARGIN + this.WIDTH, this.TOP_MARGIN - 12, 20, 10);
            this.ctx.fillRect(this.LEFT_MARGIN + this.WIDTH, this.TOP_MARGIN + this.HEIGHT + 1, 20, 10);
            
            // this.ctx.fillText("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;", this.LEFT_MARGIN + this.WIDTH, this.TOP_MARGIN + this.HEIGHT + 10);
            this.ctx.fillStyle="black"
            this.ctx.fillText(this.amplitude, this.LEFT_MARGIN + this.WIDTH, this.TOP_MARGIN - 3);
            this.ctx.fillText(-this.amplitude, this.LEFT_MARGIN + this.WIDTH, this.TOP_MARGIN + this.HEIGHT + 10);
        }
        
        if (this.draggable) {
            this.ctx.strokeStyle = this.indicatorColor;
            this.ctx.lineWidth = 2;
            drawLine(this.ctx, this.LEFT_MARGIN + this.WIDTH - 2, this.chartY(this.currentValue),
                     this.LEFT_MARGIN + this.WIDTH + 4, this.chartY(this.currentValue));
            this.ctx.lineWidth = 1;
        }
    },
        
    scroll: function() {
        var prev = this.chartY(this.prevValue);
        var current = this.chartY(this.currentValue);
        var ctx = this.ctx; // for convenience
        
        ctx.drawImage(this.canvas,
                           this.LEFT_MARGIN + 1, this.TOP_MARGIN, this.WIDTH - 1, this.HEIGHT,
                           this.LEFT_MARGIN, this.TOP_MARGIN, this.WIDTH -1, this.HEIGHT);
        // erase last item
        ctx.strokeStyle = "#fff";
        drawLine(ctx, this.LEFT_MARGIN + this.WIDTH, this.TOP_MARGIN,
                 this.LEFT_MARGIN + this.WIDTH, this.TOP_MARGIN + this.HEIGHT);
        
        // extend axis
        ctx.strokeStyle = "lightgray";
        drawLine(ctx, this.LEFT_MARGIN + this.WIDTH - 1, this.ySlideBase, this.LEFT_MARGIN + this.WIDTH, this.ySlideBase);
        
        // possibly draw a vertical line every 20 pixels
        this.vLineCount = (this.vLineCount + 1)  % this.VLINE_SEP;
        this.drawChartInterval();
        
        // and finally on to the line to be drawn
        if (this.currentValue >= -this.amplitude && this.currentValue <= this.amplitude) {
            ctx.strokeStyle = this.indicatorColor;
            drawLine(ctx, this.LEFT_MARGIN + this.WIDTH - 1, prev, this.LEFT_MARGIN + this.WIDTH, current);
        }
        if (this.draggable) {
            this.ctx.strokeStyle = "white";
            this.ctx.lineWidth = 2;
            drawLine(this.ctx, this.LEFT_MARGIN + this.WIDTH + 2, prev,
                     this.LEFT_MARGIN + this.WIDTH + 8, prev);
           
            this.ctx.strokeStyle = this.indicatorColor;
            drawLine(this.ctx, this.LEFT_MARGIN + this.WIDTH + 2, current,
                     this.LEFT_MARGIN + this.WIDTH + 8, current);
            this.ctx.lineWidth = 1;
        }
        this.prevValue = this.currentValue;
        
    }
}
